import java.io.*;
import java.util.*;
import org.chocosolver.solver.Model;
import org.chocosolver.solver.Solver;
import org.chocosolver.solver.Solution;
import org.chocosolver.solver.variables.IntVar;
import org.chocosolver.solver.search.strategy.Search;
import org.chocosolver.solver.search.strategy.strategy.IntStrategy;
import org.chocosolver.solver.search.strategy.selectors.values.IntDomainMin;
import org.chocosolver.solver.search.strategy.selectors.values.IntDomainRandom;
import org.chocosolver.solver.search.strategy.selectors.variables.InputOrder;
import org.chocosolver.solver.exception.ContradictionException;

public class Optimize {    
  
    public static void main(String[] args) throws ContradictionException, FileNotFoundException, IOException {
	JSSP jssp             = new JSSP(args[0],9999);
	String valueHeuristic = args[1];
	int timeLimit         = Integer.parseInt(args[2]);
	Model model           = jssp.model;
	Solver solver         = model.getSolver();
	Decision[] decisions  = jssp.getDecisions();
	IntVar makeSpan       = jssp.getMakeSpan();
	int lwb               = 0;
	int upb               = 9999;
	
	solver.limitTime(timeLimit*1000);
	
	if (valueHeuristic.equals("maxSlack"))
	    solver.setSearch(new IntStrategy(jssp.getDecisions(),
					     new MinSlackHeuristic(),
					     new MaxSlackValue())); 
	else if (valueHeuristic.equals("fuzzyMaxSlack"))
	    solver.setSearch(new IntStrategy(jssp.getDecisions(),
					     new MinSlackHeuristic(),
					     new FuzzyMaxSlackValue(0.5))); 
	else if (valueHeuristic.equals("minSlack"))
	    solver.setSearch(new IntStrategy(jssp.getDecisions(),
					     new MinSlackHeuristic(),
					     new MinSlackValue())); 
	else if (valueHeuristic.equals("fuzzyMinSlack"))
	    solver.setSearch(new IntStrategy(jssp.getDecisions(),
					     new MinSlackHeuristic(),
					     new FuzzyMinSlackValue(0.5))); 
	else if (valueHeuristic.equals("random"))
	    solver.setSearch(new IntStrategy(jssp.getDecisions(),
					     new MinSlackHeuristic(),
					     new FuzzyMaxSlackValue(-1.0))); 
	else solver.setSearch(Search.inputOrderLBSearch(jssp.getDecisions()));	
	//
	// attach a variable & value ordering heuristic to solver
	//
	
	model.setObjective(Model.MINIMIZE,makeSpan);
	while (solver.solve()){
	    lwb = makeSpan.getLB();
	    upb = makeSpan.getUB();
	}
	System.out.println("makespan: ["+ lwb +","+ upb +"]");
	System.out.println("nodes: "+ solver.getMeasures().getNodeCount() + 
			   "  cpu: "+ solver.getMeasures().getTimeCount());
    }
}